function readFromCache(fileName, cb, errorHandler) {
	if ( typeof errorHandler != "function")
		errorHandler = fileErrorHandler;
	readFromFile_Raw(cordova.file.cacheDirectory, fileName, function(data) {
		var dataObj = null;
		try {
			dataObj = JSON.parse(data);
			this.cb(dataObj);
		} catch(ex) {
			this.errorHandler(this.fileName, FileError.NOT_FOUND_ERR);
		}
	}.bind({
		cb : cb,
		errorHandler : errorHandler,
		fileName : fileName
	}), errorHandler);
}

function readFromFile(fileName, cb, errorHandler, readSafe) {
	if ( typeof errorHandler != "function")
		errorHandler = fileErrorHandler;
	readFromFile_Raw(cordova.file.dataDirectory, fileName, function(data) {
		try 
		{
			var jsData = JSON.parse(data); 
			cb(jsData);
			if ( typeof readSafe != "undefined")
				writeToFile(fileName + ".bak", data, errorHandler);
		} catch(ex) 
		{
			if ( ex instanceof SyntaxError && typeof readSafe != "undefined" && readSafe != false) 
			{
				readFromFile_Raw(cordova.file.dataDirectory, fileName + ".bak", function(data) 
				{
					cb(JSON.parse(data));
				}, errorHandler);
			} else
			{
				console.log(ex);
				errorHandler(fileName, ex);
			}
		}
	}, errorHandler);
}

function readFromFile_Raw(directory, fileName, cb, errorHandler) {
	if ( typeof errorHandler != "function")
		errorHandler = fileErrorHandler;
	var pathToFile = directory + fileName;
	window.resolveLocalFileSystemURL(pathToFile, function(fileEntry) {
		fileEntry.file(function(file) {
			var reader = new FileReader();

			reader.onloadend = function(e) {
				cb(this.result);
			};
			reader.readAsText(file);
		}, errorHandler.bind(null, fileName));
	}, errorHandler.bind(null, fileName));
}

var delayWriteTimer = {};
function delayWriteToCache(fileName, data, errorHandler, cb) {
	if ( typeof delayWriteTimer[fileName] == "undefined")
		delayWriteTimer[fileName] = false;
	if (delayWriteTimer[fileName])
		clearTimeout(delayWriteTimer[fileName]);

	delayWriteTimer[fileName] = setTimeout( function() {
		writeToCache(this.fileName, this.data, this.errorHandler, this.cb);
	}.bind({
		fileName : fileName,
		data : data,
		errorHandler : errorHandler,
		cb : cb
	}), 1000);
}

function writeToCache(fileName, data, errorHandler, cb) {
	if ( typeof errorHandler != "function")
		errorHandler = fileErrorHandler;
	data = JSON.stringify(data, null, '\t');

	readFromFile_Raw(cordova.file.cacheDirectory, fileName, function(oldData) {
		if (oldData != this.data)
			writeToFile_Raw(cordova.file.cacheDirectory, this.fileName, this.data, this.errorHandler, this.cb);
	}.bind({
		fileName : fileName,
		data : data
	}), function(fileName) {
		writeToFile_Raw(cordova.file.cacheDirectory, this.fileName, this.data, this.errorHandler, this.cb);
	}.bind({
		fileName : fileName,
		data : data,
		errorHandler : errorHandler,
		cb : cb
	}));
}

function writeToFile(fileName, data, errorHandler, cb) {
	if ( typeof errorHandler != "function")
		errorHandler = fileErrorHandler;
	data = JSON.stringify(data, null, '\t');
	writeToFile_Raw(cordova.file.dataDirectory, fileName, data, errorHandler, cb);
}

function writeToFile_Raw(directory, fileName, data, errorHandler, cb) {
	if ( typeof errorHandler != "function")
		errorHandler = fileErrorHandler;
	window.resolveLocalFileSystemURL(directory, function(directoryEntry) {
		directoryEntry.getFile(fileName, {
			create : true
		}, function(fileEntry) {
			fileEntry.createWriter(function(fileWriter) {
				fileWriter.onwriteend = function(e) {
					// for real-world usage, you might consider passing a success callback
					console.log('Write of file "' + directory + fileName + '" completed.');
					if ( typeof cb != "undefined")
						cb();
				};

				fileWriter.onerror = function(e) {
					// you could hook this up with our global error handler, or pass in an error callback
					console.log('Write failed: ' + e.toString());
				};

				var blob = new Blob([data], {
					type : 'text/plain'
				});
				fileWriter.write(blob);
			}, errorHandler.bind(null, fileName));
		}, errorHandler.bind(null, fileName));
	}, errorHandler.bind(null, fileName));
}

function scanDir(directory, path, errorHandler, cb) {
	if ( typeof errorHandler != "function")
		errorHandler = fileErrorHandler;
	/*TODO: No support for path yet*/
	window.resolveLocalFileSystemURL(directory, function(directoryEntry) {
		dirReader = directoryEntry.createReader();
		dirReader.readEntries(function(entries) {
			if ( typeof cb != "undefined")
				cb(entries, directoryEntry);
		}, errorHandler);
	});
}

var ajaxErrorHandler = function(jqXHR, exception) {
	$('.Loader.Freez').remove();
	if (jqXHR.status === 0) {
		alert('Not connected to Network.');
	} else if (jqXHR.status == 404) {
		alert('Requested page not found. [404]');
	} else if (jqXHR.status == 500) {
		alert('Internal Server Error [500].');
	} else if (exception === 'parsererror') {
		alert('Requested JSON parse failed.');
	} else if (exception === 'timeout') {
		alert('Time out error.');
	} else if (exception === 'abort') {
		alert('Ajax request aborted.');
	} else {
		alert('Uncaught Error.\n' + jqXHR.responseText);
	}
};
var fileErrorHandler = function(fileName, e) {
	var msg = '';

	switch (e.code) {
	case FileError.QUOTA_EXCEEDED_ERR:
		msg = 'Storage quota exceeded';
		break;
	case FileError.NOT_FOUND_ERR:
		msg = 'File not found';
		break;
	case FileError.SECURITY_ERR:
		msg = 'Security error';
		break;
	case FileError.INVALID_MODIFICATION_ERR:
		msg = 'Invalid modification';
		break;
	case FileError.INVALID_STATE_ERR:
		msg = 'Invalid state';
		break;
	default:
		msg = 'Unknown error';
		break;
	};

	alert('Error (' + fileName + '): ' + msg);
};