/******* Url Mgmt *******/
var Action = function(route)
{
	this.url = route.hasOwnProperty('url') ? route.url : null;
	this.urlForMenu = route.hasOwnProperty('urlForMenu') ? route.urlForMenu : null;
	this.route = route.hasOwnProperty('route') ? route.route : null;
	this.filePath = this.route; 
	this.needData = route.hasOwnProperty('needData') ? route.needData : true;
	this.offlineAllowed = route.hasOwnProperty('offlineAllowed') ? route.offlineAllowed : false;
	this.controlerName = this.route.replace("/", "")+"Controller"; 
	this.controler = null;
	this.id = 0;
	this.extraParams = undefined;
	this.view = null;
};

Action.prototype.PrepareView = function()
{
	var actionSegments = this.route.split("&");
	if(actionSegments.length > 1)
	{
		this.filePath = actionSegments[0];
		this.controlerName = this.filePath.replace("/", "")+"Controller";
		var pairs = actionSegments[1].split("=");
		if(pairs[0] == "id" || pairs[0] == "for")
		{
			this.id = pairs[1];
		}
		//for descreiptive test
		if(typeof actionSegments[2] != "undefined")
		{
			pairs = actionSegments[2].split("=");
			if(pairs[0] == "report")
			{
				this.extraParams = pairs[1];
			}
		}
	}
	return this.LoadController();
};

Action.prototype.LoadController = function()
{
	var defer = $.Deferred();
	TRHelper.LoadJS("controller/"+this.filePath+".js", function() {
		this.controler = eval("new " + this.controlerName + "()");
		this.controler.id = this.id;
		this.controler.urlForMenu = this.urlForMenu;
		if(typeof this.extraParams != 'undefined'){
			this.controler.report = this.extraParams;
		}
		defer.resolve();
	}.bind(this));
	return defer;
};

Action.prototype.LoadData = function()
{
	var defer = $.Deferred();
	try
	{
		if(this.needData)
		{
			DataMgmt.GetData(this.route, {}, this.offlineAllowed, function(response) {
				defer.resolve(response);
			}, function(e){
				console.log(e);
				defer.resolve({'offline': true});
			});
		}
		else
		{
			defer.resolve();
		}
	}
	catch(err)
	{
		alert(err.message);
	}
	return defer;
};

Action.prototype.LoadView = function(callback)
{	
	var viewFile = this.controler.view == "default"?"views/"+this.filePath+".html":this.controler.view;
	if(viewFile != "")
	{
		var cssFile = this.controler.style == "default"?"css/"+this.filePath+".css":this.controler.style;
		if(cssFile != "")
		{
			TRHelper.LoadCSS(cssFile);
		}
		this.view = $("<div id='" + this.controlerName + "' class='view controller-" + this.controlerName + " page' data-view-action = "+ this.route +" ></div>");
		this.view.load(viewFile, function() {
			this.callback();
		}.bind({obj:this, callback:callback}));
	}
};


var _urlMgmt = null;
var fromNotification = false;
var viewFromNotification = false;

var UrlMgmt = function(){
	this.actionStack = [];
	this.modalObj = null;
	
	this.menuAction = null;
	this.sideMenuAction = null;
	this.headerAction = null;
	
	this.menuData = null;
	this.customHeader = null;
	this.userData = null;
	this.tempUserId = null;
	
	this.showLoader = null;
	this.removeLoader = null;
	this.unlockRotation = null;
	this.lockRotation = null;
	this.lazyload = null;
	
	this.populateHeaderData = null;
	this.terminateLoading = null;
	this.debugEnabled = false;
	//bottom menu click, stack should have only home
	this.emptyStack = false;
	this.notifyUpdate = false;
	this.discussionPostType = undefined;
	this.pendingDiscussionLoad = null;
	this.subscription = "1";
	this.verifyOtp = 1;
	this.pulltorefresh = 0;
	this.menuUrlMapping = {};
	this.App = new Application();
};

UrlMgmt.getObj = function()
{
	if(_urlMgmt === null)
	{
		_urlMgmt = new UrlMgmt();
	}
	return _urlMgmt;
};
UrlMgmt.prototype.Route = function(route, callback)
{
	var pagelistcode = route.route.split("=");
	console.log("pagelistcode",pagelistcode);
	if(
		!(
			(pagelistcode[0]).includes('common/home&page') &&
			 typeof pagelistcode[1] != "undefined"       &&
			 parseInt(pagelistcode[1]) > 1)
	  )
	{
		this.showLoader(route.hasOwnProperty('needData') ? route.needData : true);
	}
	// this.showLoader(route.hasOwnProperty('needData') ? route.needData : true);
	this.StopYoutubeVideos();
	this.emptyStack = route.hasOwnProperty('emptyStack') ? route.emptyStack : false;
	this.terminateLoading = route.hasOwnProperty('terminate') ? route.terminate : false;
	var action = new Action(route);
	var prepareView = action.PrepareView();
	var loadData = action.LoadData();

	$.when(prepareView, loadData).done(function (jsloaded, data) {
		action.controler.data = data;
		if((typeof action.controler.terminate !== 'undefined' && action.controler.terminate == true) 
			|| (typeof this.terminateLoading !== 'undefined' && this.terminateLoading == true)){
			typeof callback == 'function' && callback(action);
			return;
		}
		action.LoadView(function(){
			UrlMgmt.getObj().PushView(action, function(){
				typeof callback == 'function' && callback(action);
			});
		});
	}.bind(this));
};

//To refresh only the view (As on try again)
UrlMgmt.prototype.Refresh = function(route)
{
	var toRefresh = this.actionStack[this.actionStack.length-1];
	toRefresh.needData = toRefresh.hasOwnProperty('needData') ? toRefresh.needData : true;
	toRefresh.offlineAllowed = toRefresh.hasOwnProperty('offlineAllowed') ? toRefresh.offlineAllowed : false;
	if(toRefresh.needData)
	{
		var dataMgmt = DataMgmt.GetData(toRefresh.route, {}, toRefresh.offlineAllowed, function(response) {
			toRefresh.controler.data = response;
			if(typeof toRefresh.controler.terminate == 'undefined' || toRefresh.controler.terminate == false)
			{
				toRefresh.controler.RefreshView();
			}
			}.bind(this), function(e){
				console.log(e);
				toRefresh.controler.data = {'offline': true};
				alert('Connect to Internet');
			}.bind(this));
		}
		else
		{
			if(typeof toRefresh.controler.terminate == 'undefined' || toRefresh.controler.terminate == false)
			{
				toRefresh.controler.RefreshView();
			}
	}
};

UrlMgmt.prototype.PushView = function(action, callback)
{
	if(typeof this.emptyStack !== 'undefined' && this.emptyStack == true){
		//empty the actionStack except home
		for(var i = this.actionStack.length-1; i >=1 ; i--){
			this.actionStack[i].view.remove();
			this.actionStack.pop();
		}
	}
	if(this.actionStack.length > 0){
		$(this.actionStack[this.actionStack.length-1].view).addClass('left');
	}
	if(this.actionStack.length == 0)
	{
		$('#preloader').hide();
	}
	this.actionStack.push(action);
	//Dispose video if any other page is opened
	if($(".controller-itemliveclassesController").length > 0)
	{
		UrlMgmt.prototype.disposeVideo(this.actionStack);
	}
	if(action.view != null)
	{
		$("#app").append(action.view);
	}
	if(typeof action.controler.Init !== 'undefined')
	{
		action.controler.Init(action.view);
	}
	
	callback();
};
UrlMgmt.prototype.PopView = function(callback)
{
	if (this.actionStack.length > 0) {
		var action  = this.actionStack[this.actionStack.length - 1];
		this.actionStack.pop();
		if(action.view != null)
		{
			action.view.remove();
		}
	}
	//Refresh the previous view popview
	if(this.actionStack.length > 0){
		var action = this.actionStack[this.actionStack.length-1];
		$(action.view).removeClass('left');
		this.RefreshView(action, function(){
			if(typeof callback == 'function'){
				callback();
			}
		});
	}
	else{
		if(typeof callback == 'function'){
			callback();
		}
	}
};
/* Use this , if you just want to remove the view and
don't want to refresh the view irrespective of controler refresh value */
UrlMgmt.prototype.RemoveView = function(domId)
{
	var toRemove = -1;
	if(typeof domId != 'undefined'){
		if (this.actionStack.length > 0) {
			for(var i=0; i<this.actionStack.length; i++){
				if($(this.actionStack[i].controler.dom).attr('id') == domId){
					toRemove = i;
					break;
				}
			}
		}
	}
	else{
		toRemove = this.actionStack.length - 1;
	}
	if (this.actionStack.length > 0) {
		var action  = this.actionStack[toRemove];
		this.actionStack.splice(toRemove, 1);
		if(action.view != null)
		{
			action.view.remove();
		}
	}
};
UrlMgmt.prototype.ClearActionStack = function(full)
{
	if(typeof full == 'undefined'){
		full = true;
	}
	var till = 0;
	if(full == false){
		till = 1;
	}
	for(var i = this.actionStack.length-1; i >= till; i--){
		this.actionStack[i].view.remove();
		this.actionStack.pop();
	}
};
/*Refresh the entire view depending on the refresh member variable value
 * if refresh value is false, just refresh the layout
 */
UrlMgmt.prototype.RefreshView = function(action, callback)
{
	if(typeof action.controler.refresh != 'undefined' && action.controler.refresh == true){
		$(action.view).remove();
		this.actionStack.pop();
		var route = action.route.split('&')[0];
		if(typeof action.controler.clickMenuOnRefresh != 'undefined' && action.controler.clickMenuOnRefresh == true){
			this.menuAction.view.find('ul.menu-item').find('li[data-action="'+route+'"]').click();
			if(typeof callback == 'function'){
				callback();
				return false;
			}
		}
		var urlForMenu = null;
		if(typeof action.controler.urlForMenu != 'undefined'){
			urlForMenu = action.controler.urlForMenu;
		}
		var routeObj = {route: action.route, needData: action.needData, offlineAllowed: action.offlineAllowed, urlForMenu: urlForMenu};
		UrlMgmt.getObj().Route(routeObj, function(){
			if(typeof callback == 'function'){
				callback();
			}
		});
	}
	else{
		if(typeof action.controler.SetLayout !== 'undefined'){
			action.controler.SetLayout();
		}
		if(typeof callback == 'function'){
			callback();
		}
	}
};
UrlMgmt.prototype.PullToRefresh = function(id)
{
	//threshold value to trigger -> 100px
	UrlMgmt.getObj().pulltorefresh = 1;
	var starty = 0;
	var stopy = 0;
	var scrollPos = 0;
	var appendFlag = true;
	var threshold = 100;
	var thresholdFlag = true;
	$('#'+id).bind("touchstart", function(e) {
		starty = e.originalEvent.touches[0].clientY;
		scrollPos = $('#'+id).scrollTop();
	});

	$('#'+id).bind("touchmove", function(e) {
		if(scrollPos == 0){
			stopy = e.originalEvent.changedTouches[0].clientY;
			if(stopy > starty && ((thresholdFlag && stopy - starty >=threshold) || !thresholdFlag)){
				thresholdFlag = false;
				//means scrolling down
				var diff = stopy - starty;
				if(appendFlag){
					
					var html = "<div class='refreshDiv' style='position:absolute;top:30px;width:20%;left:40%;height:30px;text-align:center;'><img src='img/refresh.svg' alt='refreshing' style='height:30px;width:30px;background-color:white;border-radius:50%;'/></div>";
					$('#app').append(html);
					appendFlag = false;
					
				}			
				if(diff <= 150 ){
					$('.refreshDiv').css('top', 40+diff-20+'px');
				}
			}
		}
	});

	$('#'+id).bind("touchend", function(e) {
		console.log('Touch end');
		//thresholdFlag = true;
		if(stopy > starty && !thresholdFlag){
			$('#app').find('.refreshDiv').css('top', '100px');
			thresholdFlag = true;
			UrlMgmt.getObj().Refresh();
			setTimeout(function(){
				//$('#'+id).css('top', '65px');
				$('#app').find('.refreshDiv').remove();
				appendFlag = true;
			}, 1000);
		}
	});
};
UrlMgmt.prototype.disposeVideo = function(actionStack){
	var routepath = actionStack[actionStack.length-1].route;
	var routesplit = routepath.split('&');
	if(routesplit[0] != "item/liveclasses")
	{
		if(typeof videojs != "undefined")
		{
			var allPlayers = videojs.getAllPlayers();
			for(var i=0;i<allPlayers.length;i++)
			{
				if($("#"+allPlayers[i].id_).length !=0)
				{
					videojs(allPlayers[i].id_).reset();
				    videojs(allPlayers[i].id_).dispose();
			   	}
			}
				
		}
		if(typeof jwplayer != "undefined")
		{	
			if($("#"+LvClass.getObj().video1InPlay).length !=0){
				jwplayer(LvClass.getObj().video1InPlay).stop();
				jwplayer(LvClass.getObj().video1InPlay).remove();
			}
			if($("#"+LvClass.getObj().video2InPlay).length !=0){
				jwplayer(LvClass.getObj().video2InPlay).stop();	
				jwplayer(LvClass.getObj().video2InPlay).remove();
			}
		}
	}
};
UrlMgmt.prototype.PopulateMenu = function(callback)
{
	if(this.menuAction == null)
	{
		this.menuAction = new Action({route:"common/tr2menu", needData:false});
		this.menuAction.PrepareView().then(function(){
			this.menuAction.LoadView(function(){
				$("#app").append(this.menuAction.view);
				this.menuAction.view.hide();
				this.menuAction.controler.Init(this.menuAction.view);
			}.bind(this));
		}.bind(this));
	}
	if(this.sideMenuAction == null)
	{
		this.sideMenuAction = new Action({route:"common/tr2sidemenu", needData:false});
		this.sideMenuAction.PrepareView().then(function(){
			this.sideMenuAction.LoadView(function(){
				$("#app").append(this.sideMenuAction.view);
				this.sideMenuAction.view.hide();
				this.sideMenuAction.controler.Init(this.sideMenuAction.view);
			}.bind(this));
		}.bind(this));
	}
	
	if(this.headerAction == null)
	{
		this.headerAction = new Action({route:"common/tr2header", needData:false});
		this.headerAction.PrepareView().then(function(){
			this.headerAction.LoadView(function(){
				$("#app").append(this.headerAction.view);
				this.headerAction.view.hide();
				this.headerAction.controler.Init(this.headerAction.view);
			}.bind(this));
		}.bind(this));
	}
};
UrlMgmt.prototype.InitiateViewProperties = function()
{
	if(this.showLoader == null){
		this.showLoader = function(needData) {
			if($('.Loader.Freez').length == 0){
				var html = '<div class="Loader Freez" data-pct="0">'+
					'<svg style="position: absolute;top: 50%;transform: translate(-50%,-50%);" viewBox="0 0 36 36">'+
						  '<path '+
								'd="M18 16 '+
							  'a 2 2 0 0 1 0 4 '+
							  'a 2 2 0 0 1 0 -4" '+
							'fill="none" '+
							'stroke="#30B8E2"; '+
							'stroke-width="0.1"; '+
							'stroke-dasharray="100, 100"'+
						  '/>'+
					'</svg>'+
					'<svg class="progress-loader" style="position: absolute;top: 50%;transform: translate(-50%,-50%);" viewBox="0 0 36 36">'+
						  '<path '+
								'd="M18 16.15 '+
							  'a 1.85 1.85 0 0 1 0 3.7 '+
							  'a 1.85 1.85 0 0 1 0 -3.7" '+
							'fill="none" '+
							'stroke="#30B8E2"; '+
							'stroke-width="0.3"; '+
							'stroke-dasharray="0, 100"'+
						  '/>'+
					'</svg></div>';
				$('body').append(html);
				if(typeof needData != 'undefined' && needData == false){
					$('.Loader .progress-loader path').attr("stroke-dasharray", "11.63, 100");
					$('.Loader').attr('data-pct', '100');
				}
				setTimeout(this.removeLoader, 50000);
			}
		};
	}
	if(this.removeLoader == null){
		this.removeLoader = function(value) {
			if (typeof value === 'undefined') {
				value = 500;
			}
			if ($('.Loader.Freez').length > 0) {
				setTimeout(function() {
					$('.Loader.Freez').remove();
				}, value);
			}
			this.lazyload && this.lazyload.update();
		};
	}
	if(this.unlockRotation == null){
		this.unlockRotation = function() {
			screen.orientation.unlock();
		};
	}
	if(this.lockRotation == null){
		this.lockRotation = function() {
			screen.orientation.lock('portrait');
		};
	}
};
UrlMgmt.prototype.StopYoutubeVideos = function()
{
	$('iframe').remove();
};
/************App**********/
var Application = function()
{
	this.isOnline = true;
	this.appId = packageName;
	this.version = {id:Appversion, type:0};
	
	this.onOnline = function()
	{
		if(this.isOnline == false){
			this.isOnline = true;
			$('.offline').removeClass('showPatti');
			$('.online').addClass('showPatti');
			$('.online').find('.dismiss-msg').unbind().bind('click', function(){
				$('.online').removeClass('showPatti');
			});
			setTimeout(function(){
				$('.online').removeClass('showPatti');
			}, 3000);
			if(UrlMgmt.getObj().menuAction != null)
				UrlMgmt.getObj().menuAction.controler.RefreshMenuData();
		}
		
		/*Auto submit the test data when user comes online if user attempted in offline mode*/
		if(typeof submitOfflineTestData === 'function')
		{
			submitOfflineTestData();
		}
		/* Load the discussions when going online */
		var urlMgmt = UrlMgmt.getObj();
		if(urlMgmt.pendingDiscussionLoad != null){
			Factory.getObj().LoadDiscussions(urlMgmt.pendingDiscussionLoad.data, urlMgmt.pendingDiscussionLoad.dom);
			urlMgmt.pendingDiscussionLoad = null;
		}
	};
	this.onOffline = function()
	{
		if(!$('.offline').hasClass('showPatti')){
			$('.online').removeClass('showPatti');
			$('.offline').addClass('showPatti');
			$('.offline').find('.dismiss-msg').unbind().bind('click', function(){
				$('.offline').removeClass('showPatti');
			});
			setTimeout(function(){
				$('.offline').removeClass('showPatti');
			}, 5000);	
		}
	};
	$(document).bind('deviceready', function(){
		try {
			var options = {
				'auto-download' : false,
				'auto-install' : false
			};
			$("#app").height(window.innerHeight-1).width(window.innerWidth);
			$("#app").height('100%');
			analytics_init();
			var urlMgmt = UrlMgmt.getObj();
			urlMgmt.InitiateViewProperties();
			urlMgmt.PopulateMenu();
			
			window.lazyLoadOptions = {};
			urlMgmt.lazyload = new LazyLoad({
				elements_selector: ".lazy",
				callback_load: function (element) {
					tr$.ImageBase64Encoding(element.src, function(dataURL, srcURL) {
						if (!(typeof window.sqlitePlugin == 'undefined' || SQLiteOps.getObj().databaseOpen == false)) {
							SQLiteOps.getObj().WriteImageBase64(srcURL, dataURL);
						}
					});
				},
				callback_error: function (element) {
					if (!(typeof window.sqlitePlugin == 'undefined' || SQLiteOps.getObj().databaseOpen == false)) {
						SQLiteOps.getObj().GetImageBase64("ImageEncodingMapping", element.src, function(count, url) {
							if (count > 0) {
								element.src = url["encodedUrl"];
							}
						});
					}
				}
			});
			
			var attachFastClick = Origami.fastclick;
			attachFastClick(document.body);

			LoggedInUser.onLoginSuccess = function()
			{
				var urlMgmt = UrlMgmt.getObj();
				urlMgmt.PopView();
				urlMgmt.Route({route:"common/home", needData:true, offlineAllowed: true});
			};
			DataMgmt.GetSavedData('config/userinfo', 1, function(response){
				if(Object.keys(response).length > 0)
			   	{
			   		var urlMgmt = UrlMgmt.getObj();
			   		urlMgmt.userData = response;
		    		urlMgmt.Route({route:"common/tr2login", needData:false}, function(action){
		    			action.controler.canskip = 1;
		    			action.controler.SkipDecision();
		    		});	
			   	}
			}, function(e){
				var urlMgmt = UrlMgmt.getObj();
				if(introScreenRequired == true){
					urlMgmt.Route({route:"common/tr2intro", needData:false});
				}
				else{
					urlMgmt.Route({route:"common/tr2login", needData:false}, function(action){
		    			action.controler.canskip = 1;
		    			action.controler.SkipDecision();
		    		});
		    	}
			});
		} catch (e) {
			// Do nothing
			console.log(e);
		} finally {
		}
		var notificationOpenedCallback = function(jsonData) {
			try{
				fromNotification = true;
				viewFromNotification = true;
				var urlKey = jsonData.notification.payload.additionalData.url;
				if (/current-affairs/i.test(urlKey)) {
					UrlMgmt.getObj().Route({
						route : 'app/currentaffairs',
						needData: false
					});
					return false;
				}
				var urls = [];
				urls.push(urlKey);
				tr$.ajax({
					mod: "tr2app",
					ack: "geturlaction",
					additionalParams:{
						urls: urls
					},
					successClb: function(response){
						var data = response.data;
						if(data.hasOwnProperty(urls[0]) && !data[urls[0]].action.includes('common/exam')){
							UrlMgmt.getObj().Route({route: data[urls[0]].action});
						}
						else{
							console.log('Action not present in app');
							createTempSession(urls[0]);
						}
					}.bind(this),
					errorClb: function(){
						//do nothing
					}
				});
			}
			catch(e){
				console.log(e);
			}
		};

		if (oneSignalRequired == true && typeof window.plugins.OneSignal != 'undefined') {
			window.plugins.OneSignal.startInit(oneSignalAppId).inFocusDisplaying(window.plugins.OneSignal.OSInFocusDisplayOption.Notification).handleNotificationOpened(notificationOpenedCallback).endInit();
		}
		
		if ( typeof window.webengage != 'undefined') {
			try {
				webengage.push.onClick(function(deeplink, customData) {
					console.log("Push clicked");
					URLRedirection(customData.url);
				});
				webengage.engage();
				if ( typeof androidfcm != "undefined") {
					androidfcm.updateToken();
					// onNewToken()
				}
			} catch(error) {
				console.log(error);
			}
		}

	});
	
	//onPluginsReady
	$(document).bind('online', this, function(e){
		this.onOnline();
	}.bind(this));
	$(document).bind('offline', this, function(e){
		e.data.isOnline = false;
		this.onOffline();
		NetworkStatusCheck.getObj().CheckStatus();
	}.bind(this));
	$(document).bind("resume", function(){
		
	});
	$(document).bind("backbutton", function(e){
		e.preventDefault();
		var urlMgmt = UrlMgmt.getObj();
		//Close the side menu, if opened
		urlMgmt.sideMenuAction.controler.CloseView();
		if(tr$.appUpdateInProgress)
		{
			//show exit confirm box if hardware back btn is pressed more than two times
			if(tr$.appUpdateInProgress >2)
			{
				if(window.confirm("Are you sure you want to exit ?"))
				{
					navigator.app.exitApp();
				}
			}
			else
			{
				tr$.appUpdateInProgress++;	
			}
			return false;
		}
		
		if(urlMgmt.modalObj)
		{
			urlMgmt.modalObj.modal('hide');
			return false;
		}
		if (viewFromNotification && urlMgmt.actionStack.length == 1){
			viewFromNotification = false;
			urlMgmt.PopView();
			urlMgmt.Route({route: 'common/home', needData: true, offlineAllowed: true});
			return false;
		}
		if (urlMgmt.actionStack.length == 1) {
			if (window.confirm("Are you sure you want to exit ?")) {
				navigator.app.exitApp();
			}
			else{
				return false;
			}
		}
		var action = urlMgmt.actionStack[urlMgmt.actionStack.length - 1];
		if ($(action.view).hasClass("controller-appssctestController") || $(action.view).hasClass("controller-appibpstestController") || $(action.view).hasClass("controller-apptbsetestController")) {
			action.view.find('.btn-back.BackFromTest').click();
		}
		else if($(action.view).hasClass("controller-appquizController")){
			action.controler.timer.seconds = 0;
			clearInterval(action.controler.timer.cancel);
			UrlMgmt.getObj().PopView();
		}
		else if($(action.view).hasClass("controller-itemliveclassesController")){
    		clearInterval(LvClass.getObj().heartBitTimerId);
			if(typeof jwplayer != 'undefined'){
				if($("#"+LvClass.getObj().video1InPlay).length !=0){
					jwplayer(LvClass.getObj().video1InPlay).stop();
					jwplayer(LvClass.getObj().video1InPlay).remove();
				}
				if($("#"+LvClass.getObj().video2InPlay).length !=0){
					jwplayer(LvClass.getObj().video2InPlay).stop();	
					jwplayer(LvClass.getObj().video2InPlay).remove();
				}
			}
			if(typeof videojs != 'undefined')
			{
				var allPlayers = videojs.getAllPlayers();
				for(var i=0;i<allPlayers.length;i++)
				{
					if($("#"+allPlayers[i].id_).length !=0)
					{
						videojs(allPlayers[i].id_).reset();
					    videojs(allPlayers[i].id_).dispose();
				   	}
				}
			}
			if(action.controler.timer != null){
				clearInterval(action.controler.timer);
			}
			UrlMgmt.getObj().PopView();
		}
		else if($(action.view).hasClass("controller-listsbatchController")){
			if(typeof action.controler.upcoming_timestamp != "undefined" && action.controler.upcoming_timestamp.length > 0)
			{
				for(var i in action.controler.upcoming_time)
				{
					clearInterval(action.controler.upcoming_time[i]);
				}
			}
			UrlMgmt.getObj().PopView();
		}
		else {
			urlMgmt.PopView();
		}
		return false;
	});
	$(document).bind("Pause", function(){
		
	});
	$(document).bind("resize", function(){
		
	});
};
